import mypkg.*;

public class main {

    public static void main(String [] args) {
        System.out.println("Welcome to the wonderful world of JR!");

	hohoho h = new hohoho(4,8);
	turkey t = new turkey();

        System.out.println(h.plus()+" "+t);

    }

}
