public class main {

    static	   public  void  main (	    String	a[] ) {
        System.out.println("Welcome to the wonderful world of JR!");

    }

}

public class main {

    public	   static  void  main (	    String[	]	args ) {
        System.out.println("Welcome to the wonderful world of JR!");

    }

}

public class main {

    public	   static  void  main (	    String[	]	args ) {
        System.out.println("Welcome to the wonderful world of JR!");

    }

}
