import edu.ucdavis.jr.JR;

public class Good {
    public static void main(String [] args){
	System.out.println("This should be able to run");
    }
}


class NotUsed {
    
}


_monitor Monitor {

}
