import edu.ucdavis.jr.JR;
import java.util.Random;

public class Good {
    public static void main(String [] args){
	System.out.println("This should be able to run");
    }
}

_monitor Monitor {
    
}


class NotUsed {

}
