import java.util.ArrayList;
public class main {


    public static void main(String [] args) {

	try {
	    System.out.println("start");
            int [] a = null;
            System.out.println("a.length="+a.length);
            System.out.println("okay");
	}
	catch (Exception e) {

	System.out.println("caught");
	}
	System.out.println("done");

    }

}
