sub u_findmain($$);
sub u_init();
sub u_jfindmain();
sub u_jgo($);
sub u_jgot(\@);
sub u_jgox($);
sub u_jr(\@);
sub u_jrc(\@);
sub u_jrfindmain();
sub u_jrgo($);
sub u_jrgot(\@);
sub u_jrgox($);
sub u_jrrun(\@);
sub u_jrt(\@);
sub u_mysystem($);
sub u_quote_args(\@);

