/*****************************************************
 * jr generated file
 ****************************************************/
import edu.ucdavis.jr.*;
import edu.ucdavis.jr.jrx.*;
import java.rmi.*;
import java.io.Serializable;

    public class JRcsp_pdecl
      extends JRjavadotlangdotObject
      implements Serializable, Cloneable
    {
	static final long serialVersionUID = 0;
	public JRcsp_pdecl(JRcsp_pdecl copy)
	{

	}
	public JRcsp_pdecl(Object ... opSig)
	{

	}
	public JRcsp_pdecl(boolean dummy)	{
	    super(dummy);

	}
	public Object clone()
	    throws CloneNotSupportedException
	{
	    return super.clone();
	}
	public static Object getNoop()
	{
	    return new JRcsp_pdecl(true);
	}
    }
