public class main2 {

    public static void main(String [] args) {
        System.out.println("Welcome to the wonderful world of JR!");

	mypkg.hohoho h = new mypkg.hohoho(4,8);
	mypkg.turkey t = new mypkg.turkey();

        System.out.println(h.plus()+" "+t);

    }

}
